import org.webbitserver.HttpHandler
import cattura.utils.io.ProcessHelper
import org.webbitserver.HttpControl
import org.webbitserver.HttpRequest
import org.webbitserver.HttpResponse
import cattura.web.server.vos.RequestResponseHelper
import cattura.web.server.interfaces.User.Role

class LogDownloaderHttpHandler implements HttpHandler
{
    private static final String URI_PREFIX = "/logs/"
    
    private static final List ALLOWED_ROLES = ["Administrator"]
    private static final String PERMISSION = "download logs"
    
    private LogDownloaderPlugin _plugin

    private File _storageDir;

    public LogDownloaderHttpHandler (final LogDownloaderPlugin $plugin, final File $storageDir)
    {
        _plugin = $plugin;
        _storageDir = $storageDir;
        
        ALLOWED_ROLES.each() {
            Role.get(it).get().grant(PERMISSION)
        }        
    }

    /*
     * HTTP interface.
     *
     * Can either accept two timestamps separated as {from}/{to} or
     * a capture session path.
     */
    public void handleHttpRequest (final HttpRequest $request, final HttpResponse $response, final HttpControl $control)
    {
        final String uri = new RequestResponseHelper($request, $response).uri();
        if (uri.startsWith(URI_PREFIX))
        {
            final String path = uri.substring(URI_PREFIX.length())
            if (_plugin.getAsyncCreatedLogArchive(path) != null)
            {
                handleHttpRequestGivenKey($response, path)
                return
            }
            
            final File captureSessionPath = new File(_storageDir, path)
            if (captureSessionPath.exists())
            {
                handleHttpRequestGivenPath($response, captureSessionPath)
            }
            else
            {
                def (from, to) = path.split("/").collect() {
                    return it.isLong() ? new Date(it as Long) : null
                }

                handleHttpRequestGivenTimestamps($response, from, to)
            }

            return
        }

        $control.nextHandler()
    }

    /*
     * /logs/{archive}.zip
     */
    private void handleHttpRequestGivenKey (final HttpResponse $response, final String $key)
    {
        final File archive = _plugin.getAsyncCreatedLogArchive($key)
        if (archive == null)
        {
            $response.status(404).content("No such log archive: ${key}").end()
            return
        }
        
        serveLogFile($response, archive, getLogFilename())
    }
    
    /*
     * /logs/{captureRequestPath}
     */
    private void handleHttpRequestGivenPath (final HttpResponse $response, final File $path)
    {
        serveLogFile($response, _plugin.createCaptureSessionLogArchive($path), "${$path.name}-logs.zip")
    }

    /*
     * /logs/{fromTimestamp}/{toTimestamp}
     */
    private void handleHttpRequestGivenTimestamps (final HttpResponse $response, Date $from, Date $to)
    {
        if (!$to)
        {
            $to = new Date()
        }

        if (!$from)
        {
            $response.status(500).content("Invalid 'from' date specified.").end()
            return
        }

        serveLogFile($response, _plugin.createLogArchive(from: $from, to: $to), getLogFilename())
    }

    private void serveLogFile (final HttpResponse $response, final File $file, final String $name)
    {
        $file.withInputStream() {
            $response.content(it.bytes)
        }
        $file.delete()
        $response.header("Content-Disposition","attachment; filename=${$name}")
                 .header("Content-Type","application/zip")
                 .end()        
    }
    
    private String getLogFilename ()
    {
        final String unitName = new ProcessHelper("uname").arg("-n").call()
        final String dateTime = new Date().toString()
        return "catturalogs_${unitName}_${dateTime}.zip".replace(" ", "_").replace(":", "-")
    }
}
